<?
/**
 * A class used in proxying remote scraping sessions.
 * See the included documentation for more details.
 */

class RemoteScrapingSession
{
	/** The commands. */
	var $CMD_NEW_SCRAPINGSESSION;
	var $CMD_DISCONNECT;
	var $CMD_SETVARIABLE;
	var $CMD_GETVARIABLE;
	var $CMD_SCRAPE;
	var $CMD_SET_LAZYSCRAPE;

	/** The delimiters. */
	var $COMMAND_DELIM;
	var $PARAM_DELIM;

	/** The prefixes. */
	var $DATARECORD_PREFIX;
	var $DATASET_PREFIX;
	var $ERROR_PREFIX;

	/** Socket used to connect to the remote server. */
	var $socket;

	/** Indicates whether or not an error occurred in the process of scraping. */
	var $is_error;

	/** Holds an error message, if one was returned. */
	var $error_message;

	/** The size of the buffer to be used when reading data from the screen-scraper server. */
	var $buffer_size;

	/** The version. */
	var $version;


	/**
	 * The constructor.
	 */
	function RemoteScrapingSession()
	{
		// Initialize the class variables.
		$this->CMD_NEW_SCRAPINGSESSION = "SSNEW";
		$this->CMD_DISCONNECT = "DISCONNECT";
		$this->CMD_SETVARIABLE = "SETVAR";
		$this->CMD_GETVARIABLE = "GETVAR";
		$this->CMD_GET_DATARECORD = "GETDR";
		$this->CMD_GET_NUM_DATARECORDS_IN_DATASET = "GETDRDS";
		$this->CMD_SCRAPE = "SCRAPE";
		$this->CMD_SET_LAZYSCRAPE = "SETLAZYSCRAPE";

		$this->COMMAND_DELIM = "#";
		$this->PARAM_DELIM = "~!~";

		$this->DATARECORD_PREFIX = "DR#";
		$this->DATASET_PREFIX = "DS#";
		$this->ERROR_PREFIX = "ER#";

		$this->is_error = false;

		$this->buffer_size = 1024;

		$this->version = "1.0";
	}


	/**
	 * Initializes the session using the given name, host, and port.
	 * Note that the host and port parameters are optional.  If they're
	 * not designated then the default values will be used.
	 *
	 * @param	name		The name of the scraping session to instantiate.
	 * @param	host		The host the session should connect to. The default
	 * is localhost.
	 * @param	port		The port the session should connect to. The default
	 * is 8778.
	 */
	function initialize( $name, $host = "localhost", $port = 8778 )
	{
		$this->init( $name, $host, $port );
	}


	/**
	 * Initializes the remote scraping session with the given name,
	 * port, and host. This method should not be called externally.
	 */
	function init( $name, $host, $port )
	{
		//echo "Opening socket...";
		//echo "Port: $port";

		$this->socket = fsockopen( $host, $port, $errno, $errstr );

		//echo "socket opened.<br>";

		if( !$this->socket )
		{
			echo "$errstr ($errno)<br>";
			exit();
		}

		$this->relayCommand( $this->CMD_NEW_SCRAPINGSESSION . $this->COMMAND_DELIM . $name );
	}


	/**
	 * Tells the remote scraping session to scrape.
	 */
	function scrape()
	{
		$this->relayCommand( $this->CMD_SCRAPE . $this->COMMAND_DELIM );
	}


	/**
	 * Gets the value of a variable from the remote scraping session.
	 *
	 * @param	var_name		The name of the desired variable.
	 * @return	The value of the variable.
	 */
	function getVariable( $var_name )
	{
		return $this->relayCommand( $this->CMD_GETVARIABLE . $this->COMMAND_DELIM . $var_name );
	}


	/**
	 * Gets the number of data records found in a data set.
	 *
	 * @param	data_set_name		The name of the data set.
	 * @return	The number of data records.
	 */
	function getNumDataRecordsInDataSet( $data_set_name )
	{
		return $this->relayCommand( $this->CMD_GET_NUM_DATARECORDS_IN_DATASET . $this->COMMAND_DELIM . $data_set_name );
	}


	/**
	 * Gets the value of a data record found in a data set from the remote scraping session.
	 *
	 * @param	data_set_name		The name of the data set.
	 * @param	index		The index of the data record.
	 * @return	The data record.
	 */
	function getDataRecordFromDataSet( $data_set_name, $index )
	{
		return $this->relayCommand( $this->CMD_GET_DATARECORD . $this->COMMAND_DELIM . $data_set_name . $this->PARAM_DELIM . $index  );
	}


	/**
	 * Sets a session variable on the remote scraping session.
	 *
	 * @param var_name		The name of the variable to be set.
	 * @param	value		The value to be set.
	 */
	function setVariable( $var_name, $value )
	{
		$this->relayCommand( $this->CMD_SETVARIABLE . $this->COMMAND_DELIM . $var_name . $this->PARAM_DELIM . $value );
	}


	/**
	 * Sets the size of the buffer to be used when reading in data 
	 * from the screen-scraper server.
	 *
	 * @param	buffer_size		The size of the buffer, in bytes.
	 */
	function setBufferSize( $buffer_size )
	{
		$this->buffer_size = $buffer_size;
	}


	/**
	 * Resets the buffer size to its default.
	 */
	function resetBufferSize()
	{
		$this->buffer_size = 1024;
	}


	/**
	 * Sends a command to the remote server.
	 *
	 * @param	commandText		The text of the command to send.
	 */
	function relayCommand( $commandText )
	{
		//echo "Sending command...";
		fputs( $this->socket, $commandText . "\n" );
		//echo "Command sent.<br>";

		$serverResponse = fgets( $this->socket, $this->buffer_size );
		//echo "Server response: $serverResponse<br>";

		if( substr( $serverResponse, 0, 3 )==$this->ERROR_PREFIX )
		{
			$this->is_error = true;
			$this->error_message = substr( $serverResponse, 3 );
		}
		elseif( substr( $serverResponse, 0, 3 )==$this->DATARECORD_PREFIX )
		{
			return $this->dataRecordStringToAssociativeArray( substr( $serverResponse, 3 ) ); 
		}
		elseif( substr( $serverResponse, 0, 3 )==$this->DATASET_PREFIX )
		{
			return $this->dataSetStringToArrayOfAssociativeArrays( substr( $serverResponse, 3 ) ); 
		}
		else
		{
			//echo "Returning: $serverResponse<br>";
			return trim( $serverResponse );
		}
	}
	

	/**
	 * Parses a string retrieved as a DataRecord from screen-scraper 
	 * into an associative array.
	 *
	 * @param	dataRecordString		The string containing the data record information.
	 * @return	The associative array.
	 */
	function dataRecordStringToAssociativeArray( $data_record_string )
	{
		//echo "Converting a data record string into an associative array.<br>";
		//echo "data_record_string: $data_record_string<br>\n";

		$params = split( "&", $data_record_string );

		foreach( $params as $param )
		{
			//echo $i . ": " . $params[$i] . "<br>\n";
			//echo "key: " . substr( $param, 0, strpos( $param, "=" ) ) . "<br>\n";
			//echo "value: " . urldecode( substr( $param, strpos( $param, "=" ) + 1 ) ) . "<br>\n";

			$return_val[substr( $param, 0, strpos( $param, "=" ) )] = urldecode( substr( $param, strpos( $param, "=" ) + 1 ) );
		}

		return $return_val;
	}
	

	/**
	 * Parses a string retrieved as a DataSet from screen-scraper 
	 * into an ordinal array of associative arrays.
	 *
	 * @param	dataSetString		The string containing the data set information.
	 * @return	The array of associative arrays.
	 */
	function dataSetStringToArrayOfAssociativeArrays( $data_set_string )
	{
		$data_records = split( " ", $data_set_string );

		for( $i = 0; $i < count( $data_records ); $i++ )
		{
			$return_val[$i] = $this->dataRecordStringToAssociativeArray( $data_records[$i] );
		}

		return $return_val;
	}


	/**
	 * Indicates whether or not an error has occurred in the scraping process.
	 *
	 * @return	A boolean.
	 */
	function isError()
	{
		return $this->is_error;
	}

	
	/**
	 * Returns the last error message returned from the server, if one was returned.
	 *
	 * @return	The error message.
	 */
	function getErrorMessage()
	{
		return $this->error_message;
	}


	/**
	 * Disconnects from the remote server.
	 */
	function disconnect()
	{
		fclose ( $this->socket );
	}


	/** 
	 * Indicates whether or not a lazy scrape should be done on the session.
	 * Defaults to false.
	 */
	function setDoLazyScrape( $doLazyScrape )
	{
		$this->relayCommand( $this->CMD_SET_LAZYSCRAPE . $this->COMMAND_DELIM . ( $doLazyScrape ? "true" : "false" ) );
	}


	/**
	 * Gets the version of the remote scraping session.
	 */
	function getVersion()
	{
		return $this->version;
	}
}
?>
